'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));

var initialState = {
  // components
  wrapperComponent: null,
  contentComponent: null,
  // settings
  limitToBounds: true,
  disabled: false,
  // position
  transformEnabled: true,
  defaultPositionX: 0,
  defaultPositionY: 0,
  positionX: 0,
  positionY: 0,
  maxPositionX: null,
  minPositionX: null,
  maxPositionY: null,
  minPositionY: null,
  // scale
  zoomingEnabled: true,
  defaultScale: 1,
  scale: 1,
  maxScale: 8,
  minScale: 0.5,
  previousScale: 1,
  // wheel
  wheelStep: 4,
  enableWheel: true,
  enableTouchPadPinch: true,
  // pinch
  pinchEnabled: true,
  pinchSensitivity: 1,
  startPinchDistance: null,
  //panning
  panningEnabled: true,
  enableZoomedOutPanning: false,
  startPanningCoords: null,
  lockAxisX: false,
  lockAxisY: false,
  // velocity
  velocityBasedOnSpeed: true,
  minVelocity: 1.6,
  enableVelocity: true,
  minVelocityScale: 1,
  // animations
  velocityAnimationSpeed: 1600,
  zoomAnimationSpeed: 200,
  // controls
  dbClickEnabled: true,
  dbClickMode: "zoomIn",
  zoomInStep: 40,
  zoomOutStep: 40,
  dbClickStep: 40
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

/**
 * Rounds number to given decimal
 * eg. roundNumber(2.34343, 1) => 2.3
 */
var roundNumber = function roundNumber(num) {
  var decimal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;

  return Number(num.toFixed(decimal));
};

/**
 * Checks if value is number, if not it returns default value
 * 1# eg. checkIsNumber(2, 30) => 2
 * 2# eg. checkIsNumber(null, 30) => 30
 */
var checkIsNumber = function checkIsNumber(num, defaultValue) {
  return typeof num === "number" ? num : defaultValue;
};

/**
 * Keeps value between given bounds, used for limiting view to given boundaries
 * 1# eg. boundLimiter(2, 0, 3, true) => 2
 * 2# eg. boundLimiter(4, 0, 3, true) => 3
 * 3# eg. boundLimiter(-2, 0, 3, true) => 0
 * 4# eg. boundLimiter(10, 0, 3, false) => 10
 */
var boundLimiter = function boundLimiter(value, minBound, maxBound, isActive) {
  if (!isActive) return roundNumber(value, 2);
  if (value < minBound) return roundNumber(minBound, 2);
  if (value > maxBound) return roundNumber(maxBound, 2);
  return roundNumber(value, 2);
};

/**
 * Calculate bounding area of zoomed/panned element
 */
var calculateBoundingArea = function calculateBoundingArea(wrapperWidth, contentWidth, diffWidth, wrapperHeight, contentHeight, diffHeight, enableZoomedOutPanning) {
  var scaleWidthFactor = wrapperWidth > contentWidth ? diffWidth * (enableZoomedOutPanning ? 1 : 0.5) : 0;
  var scaleHeightFactor = wrapperHeight > contentHeight ? diffHeight * (enableZoomedOutPanning ? 1 : 0.5) : 0;

  var minPositionX = wrapperWidth - contentWidth - scaleWidthFactor;
  var maxPositionX = 0 + scaleWidthFactor;
  var minPositionY = wrapperHeight - contentHeight - scaleHeightFactor;
  var maxPositionY = 0 + scaleHeightFactor;

  return { minPositionX: minPositionX, maxPositionX: maxPositionX, minPositionY: minPositionY, maxPositionY: maxPositionY };
};

/**
 * Returns distance between two points x,y
 */
var getDistance = function getDistance(firstPoint, secondPoint) {
  return Math.sqrt(Math.pow(firstPoint.pageX - secondPoint.pageX, 2) + Math.pow(firstPoint.pageY - secondPoint.pageY, 2));
};

/**
 * Delete undefined values from object keys
 * Used for deleting empty props
 */

var deleteUndefinedProps = function deleteUndefinedProps(value) {
  var newObject = _extends({}, value);
  Object.keys(newObject).forEach(function (key) {
    return newObject[key] == undefined && delete newObject[key];
  });
  return newObject;
};

/**
 * Fire callback if it's function
 */

var handleCallback = function handleCallback(callback, props) {
  if (callback && typeof callback === "function") {
    callback(props);
  }
};

function checkZoomBounds(zoom, minScale, maxScale) {
  if (!isNaN(maxScale) && zoom >= maxScale) return maxScale;
  if (!isNaN(minScale) && zoom <= minScale) return minScale;
  return zoom;
}

function checkPositionBounds(positionX, positionY, bounds, limitToBounds) {
  var minPositionX = bounds.minPositionX,
      minPositionY = bounds.minPositionY,
      maxPositionX = bounds.maxPositionX,
      maxPositionY = bounds.maxPositionY;

  var x = boundLimiter(positionX, minPositionX, maxPositionX, limitToBounds);
  var y = boundLimiter(positionY, minPositionY, maxPositionY, limitToBounds);
  return { x: x, y: y };
}

function getDelta(event, customDelta) {
  var deltaY = event ? event.deltaY < 0 ? 1 : -1 : 0;
  var delta = checkIsNumber(customDelta, deltaY);
  return delta;
}

function wheelMousePosition(event, contentComponent, scale) {
  var contentRect = contentComponent.getBoundingClientRect();

  // mouse position x, y over wrapper component
  var mouseX = (event.clientX - contentRect.left) / scale;
  var mouseY = (event.clientY - contentRect.top) / scale;

  if (isNaN(mouseX) || isNaN(mouseY)) return console.warn("No mouse or touch offset found");

  return {
    mouseX: mouseX,
    mouseY: mouseY
  };
}

function getComponentsSizes(wrapperComponent, contentComponent, newScale) {
  var wrapperRect = wrapperComponent.getBoundingClientRect();

  var wrapperWidth = wrapperRect.width;
  var wrapperHeight = wrapperRect.height;

  var newContentWidth = wrapperWidth * newScale;
  var newContentHeight = wrapperHeight * newScale;
  var newDiffWidth = wrapperWidth - newContentWidth;
  var newDiffHeight = wrapperHeight - newContentHeight;

  return {
    wrapperWidth: wrapperWidth,
    wrapperHeight: wrapperHeight,
    newContentWidth: newContentWidth,
    newDiffWidth: newDiffWidth,
    newContentHeight: newContentHeight,
    newDiffHeight: newDiffHeight
  };
}

function calculateZoom(zoomStep, delta, customScale) {
  var _state = this.state,
      scale = _state.scale,
      maxScale = _state.maxScale,
      minScale = _state.minScale;


  if (typeof customScale === "number" && customScale === scale) return scale;
  if (typeof customScale === "number") return checkZoomBounds(customScale, minScale, maxScale);

  var newScale = scale + zoomStep * delta * (scale / 100);

  var calculatedScale = checkZoomBounds(roundNumber(newScale, 2), minScale, maxScale);
  if (scale === calculatedScale) return scale;

  return calculatedScale;
}

function calculateTransformation(mouseX, mouseY, scaleDifference, bounds) {
  if (typeof mouseX !== "number" || typeof mouseY !== "number") return console.error("Mouse X and Y position were not provided!");
  var _state2 = this.state,
      positionX = _state2.positionX,
      positionY = _state2.positionY,
      limitToBounds = _state2.limitToBounds,
      transformEnabled = _state2.transformEnabled;


  if (!transformEnabled) return { newPositionX: positionX, newPositionY: positionY };

  var calculatedPositionX = positionX - mouseX * scaleDifference;
  var calculatedPositionY = positionY - mouseY * scaleDifference;

  var _checkPositionBounds = checkPositionBounds(calculatedPositionX, calculatedPositionY, bounds, limitToBounds),
      x = _checkPositionBounds.x,
      y = _checkPositionBounds.y;

  return { newPositionX: x, newPositionY: y };
}

function handleZoomWheel(event, customMousePosition, customDelta, customStep) {
  var _state3 = this.state,
      isDown = _state3.isDown,
      zoomingEnabled = _state3.zoomingEnabled,
      disabled = _state3.disabled,
      wrapperComponent = _state3.wrapperComponent,
      contentComponent = _state3.contentComponent,
      scale = _state3.scale,
      enableZoomedOutPanning = _state3.enableZoomedOutPanning,
      wheelStep = _state3.wheelStep;


  if (isDown || !zoomingEnabled || disabled) return;
  event.preventDefault();
  event.stopPropagation();

  // Scale transformation
  var delta = getDelta(event, customDelta);
  var newScale = calculateZoom.bind(this, customStep || wheelStep, delta)();
  if (newScale === scale) return;

  // Get new element sizes to calculate bounds

  var _getComponentsSizes = getComponentsSizes(wrapperComponent, contentComponent, newScale),
      wrapperWidth = _getComponentsSizes.wrapperWidth,
      wrapperHeight = _getComponentsSizes.wrapperHeight,
      newContentWidth = _getComponentsSizes.newContentWidth,
      newDiffWidth = _getComponentsSizes.newDiffWidth,
      newContentHeight = _getComponentsSizes.newContentHeight,
      newDiffHeight = _getComponentsSizes.newDiffHeight;

  // Position transformation


  var _ref = customMousePosition || wheelMousePosition(event, contentComponent, scale),
      mouseX = _ref.mouseX,
      mouseY = _ref.mouseY;

  var scaleDifference = newScale - scale;

  var bounds = calculateBoundingArea(wrapperWidth, newContentWidth, newDiffWidth, wrapperHeight, newContentHeight, newDiffHeight, enableZoomedOutPanning);

  // Save last zoom bounds, to speed up panning function
  this.bounds = bounds;

  // Calculate transformations

  var _calculateTransformat = calculateTransformation.bind(this, mouseX, mouseY, scaleDifference, bounds)(),
      newPositionX = _calculateTransformat.newPositionX,
      newPositionY = _calculateTransformat.newPositionY;

  this.setState({
    positionX: newPositionX,
    positionY: newPositionY,
    scale: newScale,
    previousScale: scale
  });
}

function handleZoomControls(event, customDelta, customStep) {
  var _state4 = this.state,
      positionX = _state4.positionX,
      positionY = _state4.positionY,
      wrapperComponent = _state4.wrapperComponent,
      scale = _state4.scale;
  // calculate zoom center

  var wrapperWidth = wrapperComponent.offsetWidth;
  var wrapperHeight = wrapperComponent.offsetHeight;
  var mouseX = (Math.abs(positionX) + wrapperWidth / 2) / scale;
  var mouseY = (Math.abs(positionY) + wrapperHeight / 2) / scale;
  handleZoomWheel.bind(this, event, { mouseX: mouseX, mouseY: mouseY }, customDelta, customStep)();
}

function handleZoomDbClick(event) {
  var _state5 = this.state,
      dbClickMode = _state5.dbClickMode,
      dbClickStep = _state5.dbClickStep;


  if (dbClickMode === "reset") {
    return resetTransformations.bind(this, event)();
  }
  var delta = dbClickMode === "zoomOut" ? -1 : 1;
  handleZoomWheel.bind(this, event, null, delta, dbClickStep)();
}

function resetTransformations() {
  var _props$defaultValues = this.props.defaultValues,
      defaultScale = _props$defaultValues.defaultScale,
      defaultPositionX = _props$defaultValues.defaultPositionX,
      defaultPositionY = _props$defaultValues.defaultPositionY;
  var _state6 = this.state,
      scale = _state6.scale,
      positionX = _state6.positionX,
      positionY = _state6.positionY,
      disabled = _state6.disabled;

  if (disabled) return;
  if (scale === defaultScale && positionX === defaultPositionX && positionY === defaultPositionY) return;

  var newScale = checkIsNumber(defaultScale, initialState.scale);
  var newPositionX = checkIsNumber(defaultPositionX, initialState.positionX);
  var newPositionY = checkIsNumber(defaultPositionY, initialState.positionY);
  this.setState({ scale: newScale, positionX: newPositionX, positionY: newPositionY });
}

function round(number, decimal) {
  var roundNumber$$1 = Math.pow(10, decimal);
  return Math.round(number * roundNumber$$1) / roundNumber$$1;
}

function getCurrentDistance(event) {
  return getDistance(event.touches[0], event.touches[1]);
}

function checkIfInfinite(number) {
  return number === Infinity || number === -Infinity;
}

function calculatePinchZoom(currentDistance, delta, pinchStartDistance) {
  var _state = this.state,
      minScale = _state.minScale,
      maxScale = _state.maxScale;

  if (typeof pinchStartDistance !== "number" || typeof currentDistance !== "number") return console.error("Pinch touches distance was not provided");

  if (currentDistance < 0) return;
  var touchProportion = currentDistance / pinchStartDistance;
  var scaleDifference = touchProportion * this.pinchStartScale;

  return checkZoomBounds(scaleDifference, minScale, maxScale);
}

function calculateMidpoint(event, scale, contentComponent) {
  var contentRect = contentComponent.getBoundingClientRect();
  var touches = event.touches;

  var firstPointX = round(touches[0].clientX - contentRect.left, 5);
  var firstPointY = round(touches[0].clientY - contentRect.top, 5);
  var secondPointX = round(touches[1].clientX - contentRect.left, 5);
  var secondPointY = round(touches[1].clientY - contentRect.top, 5);

  return {
    mouseX: (firstPointX + secondPointX) / 2 / scale,
    mouseY: (firstPointY + secondPointY) / 2 / scale
  };
}

function handleZoomPinch(event) {
  var _state2 = this.state,
      isDown = _state2.isDown,
      zoomingEnabled = _state2.zoomingEnabled,
      disabled = _state2.disabled,
      wrapperComponent = _state2.wrapperComponent,
      contentComponent = _state2.contentComponent,
      scale = _state2.scale,
      enableZoomedOutPanning = _state2.enableZoomedOutPanning;

  if (isDown || !zoomingEnabled || disabled) return;

  event.preventDefault();
  event.stopPropagation();

  // Position transformation

  var _calculateMidpoint = calculateMidpoint(event, scale, contentComponent),
      mouseX = _calculateMidpoint.mouseX,
      mouseY = _calculateMidpoint.mouseY;

  // if touches goes off of the wrapper element


  if (checkIfInfinite(mouseX) || checkIfInfinite(mouseY)) return;

  var currentDistance = getCurrentDistance(event);
  var delta = currentDistance < this.lastDistance ? 1 : -1;

  var newScale = calculatePinchZoom.bind(this, currentDistance, delta, this.pinchStartDistance)();
  if (checkIfInfinite(newScale) || newScale === scale) return;

  var scaleDifference = newScale - scale;

  // Get new element sizes to calculate bounds

  var _getComponentsSizes = getComponentsSizes(wrapperComponent, contentComponent, newScale),
      wrapperWidth = _getComponentsSizes.wrapperWidth,
      wrapperHeight = _getComponentsSizes.wrapperHeight,
      newContentWidth = _getComponentsSizes.newContentWidth,
      newDiffWidth = _getComponentsSizes.newDiffWidth,
      newContentHeight = _getComponentsSizes.newContentHeight,
      newDiffHeight = _getComponentsSizes.newDiffHeight;

  var bounds = calculateBoundingArea(wrapperWidth, newContentWidth, newDiffWidth, wrapperHeight, newContentHeight, newDiffHeight, enableZoomedOutPanning);

  // Save last zoom bounds, to speed up panning function
  this.bounds = bounds;

  // Calculate transformations

  var _calculateTransformat = calculateTransformation.bind(this, mouseX, mouseY, scaleDifference, bounds)(),
      newPositionX = _calculateTransformat.newPositionX,
      newPositionY = _calculateTransformat.newPositionY;

  this.lastDistance = currentDistance;

  this.setState({
    positionX: newPositionX,
    positionY: newPositionY,
    scale: newScale,
    previousScale: scale
  });
}

function getSizes(wrapperComponent, contentComponent) {
  var wrapperRect = wrapperComponent.getBoundingClientRect();
  var contentRect = contentComponent.getBoundingClientRect();

  var wrapperWidth = wrapperRect.width;
  var wrapperHeight = wrapperRect.height;
  var contentWidth = contentRect.width;
  var contentHeight = contentRect.height;
  var diffWidth = wrapperWidth - contentWidth;
  var diffHeight = wrapperHeight - contentHeight;

  return {
    wrapperWidth: wrapperWidth,
    wrapperHeight: wrapperHeight,
    contentWidth: contentWidth,
    diffWidth: diffWidth,
    contentHeight: contentHeight,
    diffHeight: diffHeight
  };
}

function handleCalculateBounds(wrapperComponent, contentComponent, enableZoomedOutPanning) {
  var _getSizes = getSizes(wrapperComponent, contentComponent),
      wrapperWidth = _getSizes.wrapperWidth,
      contentWidth = _getSizes.contentWidth,
      diffWidth = _getSizes.diffWidth,
      wrapperHeight = _getSizes.wrapperHeight,
      contentHeight = _getSizes.contentHeight,
      diffHeight = _getSizes.diffHeight;

  return calculateBoundingArea(wrapperWidth, contentWidth, diffWidth, wrapperHeight, contentHeight, diffHeight, enableZoomedOutPanning);
}

function getClientPosition(event) {
  var touches = event.touches;

  // Mobile points

  if (touches && touches.length === 1) {
    return { clientX: touches[0].clientX, clientY: touches[0].clientY };
  }
  // Desktop points
  if (!touches) {
    return { clientX: event.clientX, clientY: event.clientY };
  }
  return null;
}

function handlePanning(event) {
  var _state = this.state,
      wrapperComponent = _state.wrapperComponent,
      contentComponent = _state.contentComponent,
      enableZoomedOutPanning = _state.enableZoomedOutPanning,
      limitToBounds = _state.limitToBounds,
      positionX = _state.positionX,
      positionY = _state.positionY,
      scale = _state.scale,
      lockAxisX = _state.lockAxisX,
      lockAxisY = _state.lockAxisY;


  if (!this.startCoords || scale === 1) return;
  var _startCoords = this.startCoords,
      x = _startCoords.x,
      y = _startCoords.y;


  var positions = getClientPosition(event);
  if (!positions) return console.error("Cannot find mouse client positions");
  var clientX = positions.clientX,
      clientY = positions.clientY;

  // Calculate bounding area

  var bounds = this.bounds;

  // If panning is fired before scaling and there are no bounds calculated
  if (!bounds) {
    var calculatedBounds = handleCalculateBounds(wrapperComponent, contentComponent, enableZoomedOutPanning);
    this.bounds = calculatedBounds;
    bounds = calculatedBounds;
  }

  // Get Position
  var newPositionX = lockAxisX ? positionX : clientX - x;
  var newPositionY = lockAxisY ? positionY : clientY - y;

  // If position didn't change
  if (newPositionX === positionX && newPositionY === positionY) return;

  var calculatedPosition = checkPositionBounds(newPositionX, newPositionY, bounds, limitToBounds);

  // Save panned position
  this.setState({ positionX: calculatedPosition.x, positionY: calculatedPosition.y });
}

// We want to make event listeners non-passive, and to do so have to check

function makePassiveEventOption(passive) {
  return passive;
}

var Context = React__default.createContext({});

var timer = null;
var timerTime = 50;

var StateProvider = function (_Component) {
  inherits(StateProvider, _Component);

  function StateProvider() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, StateProvider);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = StateProvider.__proto__ || Object.getPrototypeOf(StateProvider)).call.apply(_ref, [this].concat(args))), _this), _this.state = _extends({}, initialState, _this.props.defaultValues, _this.props.dynamicValues, {
      previousScale: initialState.scale,
      startAnimation: false
    }), _this.pinchStartDistance = null, _this.lastDistance = null, _this.pinchStartScale = null, _this.distance = null, _this.bounds = null, _this.startPanningCoords = null, _this.velocityTime = null, _this.lastMousePosition = null, _this.velocity = null, _this.animate = false, _this.offsetX = null, _this.offsetY = null, _this.handleWheel = function (event) {
      var _this$state = _this.state,
          enableWheel = _this$state.enableWheel,
          enableTouchPadPinch = _this$state.enableTouchPadPinch;
      var _this$props = _this.props,
          onWheelStart = _this$props.onWheelStart,
          onWheel = _this$props.onWheel,
          onWheelStop = _this$props.onWheelStop;

      // ctrlKey detects if touchpad execute wheel or pinch gesture

      if (!enableWheel && !event.ctrlKey) return;
      if (!enableTouchPadPinch && event.ctrlKey) return;

      if (!timer) {
        // Wheel start event
        _this.handleDisableVelocity();
        handleCallback(onWheelStart, _this.getCallbackProps());
      }

      // Wheel event
      handleZoomWheel.bind(_this, event)();
      handleCallback(onWheel, _this.getCallbackProps());

      // Wheel stop event
      clearTimeout(timer);
      timer = setTimeout(function () {
        handleCallback(onWheelStop, _this.getCallbackProps());
        timer = null;
      }, timerTime);
    }, _this.checkIsPanningActive = function (event) {
      var _this$state2 = _this.state,
          panningEnabled = _this$state2.panningEnabled,
          disabled = _this$state2.disabled;

      return !_this.isDown || !panningEnabled || disabled || event.touches && (event.touches.length !== 1 || Math.abs(_this.startCoords.x - event.touches[0].clientX) < 1);
    }, _this.handleSetUpPanning = function (x, y) {
      var _this$state3 = _this.state,
          positionX = _this$state3.positionX,
          positionY = _this$state3.positionY;

      _this.isDown = true;
      _this.startCoords = { x: x - positionX, y: y - positionY };
      handleCallback(_this.props.onPanningStart, _this.getCallbackProps());
    }, _this.handleStartPanning = function (event) {
      var _this$state4 = _this.state,
          panningEnabled = _this$state4.panningEnabled,
          disabled = _this$state4.disabled,
          wrapperComponent = _this$state4.wrapperComponent;
      var target = event.target,
          touches = event.touches;

      if (!panningEnabled || disabled || !wrapperComponent.contains(target)) return;

      _this.handleDisableVelocity();
      // Mobile points
      if (touches && touches.length === 1) {
        _this.handleSetUpPanning(touches[0].clientX, touches[0].clientY);
      }
      // Desktop points
      if (!touches) {
        _this.handleSetUpPanning(event.clientX, event.clientY);
      }
    }, _this.handlePanning = function (event) {
      event.preventDefault();
      if (_this.checkIsPanningActive(event)) return;
      event.stopPropagation();

      _this.calculateVelocityStart(event);
      handlePanning.bind(_this, event)();
      handleCallback(_this.props.onPanning, _this.getCallbackProps());
    }, _this.handleStopPanning = function () {
      if (_this.isDown) {
        _this.isDown = false;
        _this.handleFireVelocity();
        handleCallback(_this.props.onPanningStop, _this.getCallbackProps());
      }
    }, _this.handlePinchStart = function (event) {
      var scale = _this.state.scale;

      event.preventDefault();
      event.stopPropagation();

      _this.handleDisableVelocity();
      var distance = getDistance(event.touches[0], event.touches[1]);
      _this.pinchStartDistance = distance;
      _this.lastDistance = distance;
      _this.pinchStartScale = scale;

      handleCallback(_this.props.onPinchingStart, _this.getCallbackProps());
    }, _this.handlePinch = function (event) {
      handleZoomPinch.bind(_this, event)();
      handleCallback(_this.props.onPinching, _this.getCallbackProps());
    }, _this.handlePinchStop = function () {
      if (typeof pinchStartScale === "number") {
        _this.pinchStartDistance = null;
        _this.lastDistance = null;
        _this.pinchStartScale = null;
        handleCallback(_this.props.onPinchingStop, _this.getCallbackProps());
      }
    }, _this.handleTouchStart = function (event) {
      var disabled = _this.state.disabled;
      var touches = event.touches;

      _this.handleDisableVelocity();
      if (disabled) return;
      if (touches && touches.length === 1) return _this.handleStartPanning(event);
      if (touches && touches.length === 2) return _this.handlePinchStart(event);
    }, _this.handleTouch = function (event) {
      var _this$state5 = _this.state,
          panningEnabled = _this$state5.panningEnabled,
          pinchEnabled = _this$state5.pinchEnabled,
          disabled = _this$state5.disabled;

      if (disabled) return;
      if (panningEnabled && event.touches.length === 1) return _this.handlePanning(event);
      if (pinchEnabled && event.touches.length === 2) return _this.handlePinch(event);
    }, _this.handleTouchStop = function () {
      _this.handlePinchStop();
      _this.handleStopPanning();
    }, _this.velocityTimeSpeed = function (speed, animationTime) {
      var velocityBasedOnSpeed = _this.state.velocityBasedOnSpeed;


      if (velocityBasedOnSpeed) {
        return animationTime - animationTime / Math.max(1.6, speed);
      }
      return animationTime;
    }, _this.handleEnableVelocity = function () {
      _this.setState({ startAnimation: false });
    }, _this.handleDisableVelocity = function () {
      _this.velocity = null;
      _this.animate = false;
      _this.setState({ startAnimation: false });
    }, _this.handleFireVelocity = function () {
      _this.setState({ startAnimation: true });
    }, _this.easeOut = function (p) {
      return -Math.cos(p * Math.PI) / 2 + 0.5;
    }, _this.animateVelocity = function () {
      _this.animate = true;
      var _this$state6 = _this.state,
          startAnimation = _this$state6.startAnimation,
          positionX = _this$state6.positionX,
          positionY = _this$state6.positionY,
          limitToBounds = _this$state6.limitToBounds,
          scale = _this$state6.scale,
          velocityAnimationSpeed = _this$state6.velocityAnimationSpeed,
          lockAxisX = _this$state6.lockAxisX,
          lockAxisY = _this$state6.lockAxisY;

      var startTime = new Date().getTime();

      if (!_this.velocity || !_this.bounds) return _this.handleDisableVelocity();
      var _this$velocity = _this.velocity,
          velocityX = _this$velocity.velocityX,
          velocityY = _this$velocity.velocityY,
          velocity = _this$velocity.velocity;

      var animationTime = _this.velocityTimeSpeed(velocity, velocityAnimationSpeed);
      var targetX = velocityX * scale;
      var targetY = velocityY * scale;

      _this.offsetX = positionX;
      _this.offsetY = positionY;

      var animate = function animate() {
        if (!startAnimation || !_this.animate) {
          return;
        }

        var frameTime = new Date().getTime() - startTime;
        var time = frameTime / animationTime;
        var step = _this.easeOut(time);
        if (frameTime >= animationTime) {
          _this.handleDisableVelocity();
        } else {
          var currentPositionX = lockAxisX ? positionX : _this.offsetX + targetX - targetX * step;
          var currentPositionY = lockAxisY ? positionY : _this.offsetY + targetY - targetY * step;

          var calculatedPosition = checkPositionBounds(currentPositionX, currentPositionY, _this.bounds, limitToBounds);

          _this.offsetX = calculatedPosition.x;
          _this.offsetY = calculatedPosition.y;

          // Save panned position
          _this.setState({ positionX: calculatedPosition.x, positionY: calculatedPosition.y });
          requestAnimationFrame(animate);
        }
      };
      requestAnimationFrame(animate);
    }, _this.calculateVelocityStart = function (event) {
      var _this$state7 = _this.state,
          enableVelocity = _this$state7.enableVelocity,
          minVelocityScale = _this$state7.minVelocityScale,
          scale = _this$state7.scale,
          disabled = _this$state7.disabled;

      if (!enableVelocity || minVelocityScale >= scale || disabled) return;
      _this.handleEnableVelocity();
      var now = Date.now();
      if (_this.lastMousePosition) {
        var _position = getClientPosition(event);
        if (!_position) return console.error("No mouse or touch position detected");
        var clientX = _position.clientX,
            clientY = _position.clientY;

        var distanceX = clientX - _this.lastMousePosition.clientX;
        var distanceY = clientY - _this.lastMousePosition.clientY;
        var interval = now - _this.velocityTime;
        var velocityX = distanceX / interval;
        var velocityY = distanceY / interval;
        var velocity = Math.sqrt(distanceX * distanceX + distanceY * distanceY) / interval;

        _this.velocity = { velocityX: velocityX, velocityY: velocityY, velocity: velocity };
      }
      var position = getClientPosition(event);
      _this.lastMousePosition = position;
      _this.velocityTime = now;
    }, _this.resetLastMousePosition = function () {
      return _this.setState({ lastMouseEventPosition: null });
    }, _this.zoomIn = function (event) {
      var _this$state8 = _this.state,
          zoomingEnabled = _this$state8.zoomingEnabled,
          disabled = _this$state8.disabled,
          zoomInStep = _this$state8.zoomInStep;

      if (!event) throw Error("Zoom in function require event prop");
      if (!zoomingEnabled || disabled) return;
      handleZoomControls.bind(_this, event, 1, zoomInStep)();
    }, _this.zoomOut = function (event) {
      var _this$state9 = _this.state,
          zoomingEnabled = _this$state9.zoomingEnabled,
          disabled = _this$state9.disabled,
          zoomOutStep = _this$state9.zoomOutStep;

      if (!event) throw Error("Zoom out function require event prop");
      if (!zoomingEnabled || disabled) return;
      handleZoomControls.bind(_this, event, -1, zoomOutStep)();
    }, _this.handleDbClick = function (event) {
      var _this$state10 = _this.state,
          zoomingEnabled = _this$state10.zoomingEnabled,
          disabled = _this$state10.disabled,
          dbClickStep = _this$state10.dbClickStep,
          dbClickEnabled = _this$state10.dbClickEnabled;

      if (!event) throw Error("Double click function require event prop");
      if (!zoomingEnabled || disabled || !dbClickEnabled) return;
      handleZoomDbClick.bind(_this, event, 1, dbClickStep)();
    }, _this.setScale = function (scale) {
      _this.setState({ scale: scale });
    }, _this.setPositionX = function (positionX) {
      _this.setState({ positionX: roundNumber(positionX, 3) });
    }, _this.setPositionY = function (positionY) {
      _this.setState({ positionY: roundNumber(positionY, 3) });
    }, _this.setTransform = function (positionX, positionY, scale) {
      if (!_this.state.transformEnabled) return;
      !isNaN(scale) && _this.setScale(scale);
      !isNaN(positionX) && _this.setPositionX(positionX);
      !isNaN(positionY) && _this.setPositionY(positionY);
    }, _this.resetTransform = function () {
      var disabled = _this.state.disabled;

      if (disabled) return;
      resetTransformations.bind(_this)();
    }, _this.setWrapperComponent = function (wrapperComponent) {
      _this.setState({ wrapperComponent: wrapperComponent });
    }, _this.setContentComponent = function (contentComponent) {
      _this.setState({ contentComponent: contentComponent });
    }, _this.getCallbackProps = function () {
      return {
        positionX: _this.state.positionX,
        positionY: _this.state.positionY,
        scale: _this.state.scale,
        sensitivity: _this.state.sensitivity,
        maxScale: _this.state.maxScale,
        minScale: _this.state.minScale,
        wheelAnimationSpeed: _this.state.wheelAnimationSpeed,
        zoomAnimationSpeed: _this.state.zoomAnimationSpeed,
        pinchAnimationSpeed: _this.state.pinchAnimationSpeed,
        panAnimationSpeed: _this.state.panAnimationSpeed,
        minPositionX: _this.state.minPositionX,
        minPositionY: _this.state.minPositionY,
        maxPositionX: _this.state.maxPositionX,
        maxPositionY: _this.state.maxPositionY,
        limitToBounds: _this.state.limitToBounds,
        zoomingEnabled: _this.state.zoomingEnabled,
        panningEnabled: _this.state.panningEnabled,
        transformEnabled: _this.state.transformEnabled,
        pinchEnabled: _this.state.pinchEnabled,
        enableZoomedOutPanning: _this.state.enableZoomedOutPanning,
        disabled: _this.state.disabled,
        zoomOutStep: _this.state.zoomOutStep,
        zoomInStep: _this.state.zoomInStep,
        dbClickStep: _this.state.dbClickStep,
        pinchSensitivity: _this.state.pinchSensitivity,
        dbClickEnabled: _this.state.dbClickEnabled,
        lastPositionZoomEnabled: _this.state.lastPositionZoomEnabled,
        previousScale: _this.state.previousScale
      };
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(StateProvider, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;

      var passiveOption = makePassiveEventOption(false);

      // Panning on window to allow panning when mouse is out of wrapper
      window.addEventListener("mousedown", this.handleStartPanning, passiveOption);
      window.addEventListener("mousemove", this.handlePanning, passiveOption);
      window.addEventListener("mouseup", this.handleStopPanning, passiveOption);
      return function () {
        window.removeEventListener("mousedown", _this2.handleStartPanning, passiveOption);
        window.removeEventListener("mousemove", _this2.handlePanning, passiveOption);
        window.removeEventListener("mouseup", _this2.handleStopPanning, passiveOption);
      };
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(oldProps, oldState) {
      var _state = this.state,
          wrapperComponent = _state.wrapperComponent,
          enableZoomedOutPanning = _state.enableZoomedOutPanning,
          startAnimation = _state.startAnimation;
      var dynamicValues = this.props.dynamicValues;

      if (!oldState.wrapperComponent && this.state.wrapperComponent) {
        // Zooming events on wrapper
        var passiveOption = makePassiveEventOption(false);
        wrapperComponent.addEventListener("mousewheel", this.handleWheel, passiveOption);
        wrapperComponent.addEventListener("dblclick", this.handleDbClick, passiveOption);
        wrapperComponent.addEventListener("touchstart", this.handleTouchStart, passiveOption);
        wrapperComponent.addEventListener("touchmove", this.handleTouch, passiveOption);
        wrapperComponent.addEventListener("touchend", this.handleTouchStop, passiveOption);
      }
      // eslint-disable-next-line react/no-did-update-set-state
      if (oldProps.dynamicValues !== dynamicValues) this.setState(_extends({}, dynamicValues));

      if (this.bounds && oldState.enableZoomedOutPanning !== enableZoomedOutPanning) this.bounds = null;
      if (this.velocity && startAnimation && !this.animate) this.animateVelocity();
    }

    //////////
    // Wheel
    //////////

    //////////
    // Panning
    //////////

    //////////
    // Pinch
    //////////

    //////////
    // Touch Events
    //////////

    //////////
    // Velocity
    //////////

    //////////
    // Controls
    //////////

    //////////
    // Setters
    //////////

    //////////
    // Props
    //////////

  }, {
    key: "render",
    value: function render() {
      /**
       * Context provider value
       */
      var value = {
        state: this.getCallbackProps(),
        dispatch: {
          setScale: this.setScale,
          setPositionX: this.setPositionX,
          setPositionY: this.setPositionY,
          zoomIn: this.zoomIn,
          zoomOut: this.zoomOut,
          setTransform: this.setTransform,
          resetTransform: this.resetTransform
        },
        nodes: {
          setWrapperComponent: this.setWrapperComponent,
          setContentComponent: this.setContentComponent
        },
        internal: {
          handleZoom: this.handleZoom,
          handleStartPanning: this.handleStartPanning,
          handlePanning: this.handlePanning,
          handleStopPanning: this.handleStopPanning,
          handleDbClick: this.handleDbClick,
          handleTouchStart: this.handleTouchStart,
          handleTouch: this.handleTouch,
          handleTouchStop: this.handleTouchStop
        }
      };
      var children = this.props.children;

      var content = typeof children === "function" ? children(_extends({}, value.state, value.dispatch)) : children;

      return React__default.createElement(
        Context.Provider,
        { value: value },
        content
      );
    }
  }]);
  return StateProvider;
}(React.Component);

StateProvider.defaultProps = {
  defaultValues: {},
  dynamicValues: {},
  onWheelStart: null,
  onWheel: null,
  onWheelStop: null,
  onPanningStart: null,
  onPanning: null,
  onPanningStop: null,
  onPinchingStart: null,
  onPinching: null,
  onPinchingStop: null
};

StateProvider.propTypes = {
  children: PropTypes.any,
  defaultValues: PropTypes.object,
  dynamicValues: PropTypes.object,
  onWheelStart: PropTypes.func,
  onWheel: PropTypes.func,
  onWheelStop: PropTypes.func,
  onPanningStart: PropTypes.func,
  onPanning: PropTypes.func,
  onPanningStop: PropTypes.func,
  onPinchingStart: PropTypes.func,
  onPinching: PropTypes.func,
  onPinchingStop: PropTypes.func
};

var TransformWrapper = function TransformWrapper(_ref) {
  var children = _ref.children,
      scale = _ref.scale,
      positionX = _ref.positionX,
      positionY = _ref.positionY,
      sensitivity = _ref.sensitivity,
      maxScale = _ref.maxScale,
      minScale = _ref.minScale,
      wheelAnimationSpeed = _ref.wheelAnimationSpeed,
      zoomAnimationSpeed = _ref.zoomAnimationSpeed,
      pinchAnimationSpeed = _ref.pinchAnimationSpeed,
      panAnimationSpeed = _ref.panAnimationSpeed,
      resetAnimationSpeed = _ref.resetAnimationSpeed,
      minPositionX = _ref.minPositionX,
      minPositionY = _ref.minPositionY,
      maxPositionX = _ref.maxPositionX,
      maxPositionY = _ref.maxPositionY,
      limitToBounds = _ref.limitToBounds,
      zoomingEnabled = _ref.zoomingEnabled,
      panningEnabled = _ref.panningEnabled,
      transformEnabled = _ref.transformEnabled,
      pinchEnabled = _ref.pinchEnabled,
      enableZoomedOutPanning = _ref.enableZoomedOutPanning,
      disabled = _ref.disabled,
      zoomOutStep = _ref.zoomOutStep,
      zoomInStep = _ref.zoomInStep,
      dbClickStep = _ref.dbClickStep,
      pinchSensitivity = _ref.pinchSensitivity,
      dbClickEnabled = _ref.dbClickEnabled,
      lastPositionZoomEnabled = _ref.lastPositionZoomEnabled,
      enableZoomThrottling = _ref.enableZoomThrottling,
      onWheelStart = _ref.onWheelStart,
      onWheel = _ref.onWheel,
      onWheelStop = _ref.onWheelStop,
      onPanningStart = _ref.onPanningStart,
      onPanning = _ref.onPanning,
      onPanningStop = _ref.onPanningStop,
      onPinchingStart = _ref.onPinchingStart,
      onPinching = _ref.onPinching,
      onPinchingStop = _ref.onPinchingStop,
      defaultPositionX = _ref.defaultPositionX,
      defaultPositionY = _ref.defaultPositionY,
      defaultScale = _ref.defaultScale;

  return React__default.createElement(
    StateProvider,
    {
      defaultValues: deleteUndefinedProps({
        defaultPositionX: defaultPositionX,
        defaultPositionY: defaultPositionY,
        defaultScale: defaultScale
      }),
      dynamicValues: deleteUndefinedProps({
        scale: scale,
        positionX: positionX,
        positionY: positionY,
        sensitivity: sensitivity,
        maxScale: maxScale,
        minScale: minScale,
        wheelAnimationSpeed: wheelAnimationSpeed,
        zoomAnimationSpeed: zoomAnimationSpeed,
        pinchAnimationSpeed: pinchAnimationSpeed,
        panAnimationSpeed: panAnimationSpeed,
        resetAnimationSpeed: resetAnimationSpeed,
        minPositionX: minPositionX,
        minPositionY: minPositionY,
        maxPositionX: maxPositionX,
        maxPositionY: maxPositionY,
        limitToBounds: limitToBounds,
        zoomingEnabled: zoomingEnabled,
        panningEnabled: panningEnabled,
        transformEnabled: transformEnabled,
        pinchEnabled: pinchEnabled,
        enableZoomedOutPanning: enableZoomedOutPanning,
        disabled: disabled,
        zoomOutStep: zoomOutStep,
        zoomInStep: zoomInStep,
        dbClickStep: dbClickStep,
        pinchSensitivity: pinchSensitivity,
        dbClickEnabled: dbClickEnabled,
        lastPositionZoomEnabled: lastPositionZoomEnabled,
        enableZoomThrottling: enableZoomThrottling
      }),
      onWheelStart: onWheelStart,
      onWheel: onWheel,
      onWheelStop: onWheelStop,
      onPanningStart: onPanningStart,
      onPanning: onPanning,
      onPanningStop: onPanningStop,
      onPinchingStart: onPinchingStart,
      onPinching: onPinching,
      onPinchingStop: onPinchingStop
    },
    children
  );
};

TransformWrapper.propTypes = {
  children: PropTypes.any,
  scale: PropTypes.number,
  positionX: PropTypes.number,
  positionY: PropTypes.number,
  sensitivity: PropTypes.number,
  maxScale: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  minScale: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  wheelAnimationSpeed: PropTypes.number,
  zoomAnimationSpeed: PropTypes.number,
  pinchAnimationSpeed: PropTypes.number,
  panAnimationSpeed: PropTypes.number,
  resetAnimationSpeed: PropTypes.number,
  minPositionX: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  maxPositionX: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  minPositionY: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  maxPositionY: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  limitToBounds: PropTypes.bool,
  zoomingEnabled: PropTypes.bool,
  panningEnabled: PropTypes.bool,
  transformEnabled: PropTypes.bool,
  pinchEnabled: PropTypes.bool,
  enableZoomedOutPanning: PropTypes.bool,
  disabled: PropTypes.bool,
  zoomOutStep: PropTypes.bool,
  zoomInStep: PropTypes.bool,
  dbClickStep: PropTypes.bool,
  pinchSensitivity: PropTypes.bool,
  dbClickEnabled: PropTypes.bool,
  lastPositionZoomEnabled: PropTypes.bool,
  enableZoomThrottling: PropTypes.bool,
  onWheelStart: PropTypes.func,
  onWheel: PropTypes.func,
  onWheelStop: PropTypes.func,
  onPanningStart: PropTypes.func,
  onPanning: PropTypes.func,
  onPanningStop: PropTypes.func,
  onPinchingStart: PropTypes.func,
  onPinching: PropTypes.func,
  onPinchingStop: PropTypes.func,
  defaultPositionX: PropTypes.number,
  defaultPositionY: PropTypes.number,
  defaultScale: PropTypes.number
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".TransformComponent-module_container__3NwNd {\n  position: relative;\n  width: fit-content;\n  height: fit-content;\n  overflow: hidden;\n  -webkit-touch-callout: none; /* iOS Safari */\n  -webkit-user-select: none; /* Safari */\n  -khtml-user-select: none; /* Konqueror HTML */\n  -moz-user-select: none; /* Firefox */\n  -ms-user-select: none; /* Internet Explorer/Edge */\n  user-select: none;\n  margin: 0;\n  padding: 0;\n}\n.TransformComponent-module_content__TZU5O {\n  display: flex;\n  flex-wrap: wrap;\n  width: fit-content;\n  height: fit-content;\n  transform-origin: 0% 0%;\n  margin: 0;\n  padding: 0;\n}\n.TransformComponent-module_content__TZU5O img {\n  pointer-events: none;\n}\n";
var styles = { "container": "TransformComponent-module_container__3NwNd", "content": "TransformComponent-module_content__TZU5O" };
styleInject(css);

function TransformComponent(_ref) {
  var children = _ref.children;

  var wrapperRef = React.useRef(null);
  var contentRef = React.useRef(null);

  var _useContext = React.useContext(Context),
      state = _useContext.state,
      nodes = _useContext.nodes,
      internal = _useContext.internal;

  var style = {
    transform: "translate(" + state.positionX + "px, " + state.positionY + "px) scale(" + state.scale + ")"
  };

  React.useEffect(function () {
    if (wrapperRef.current) {
      nodes.setWrapperComponent(wrapperRef.current);
    }
    if (contentRef.current) {
      nodes.setContentComponent(contentRef.current);
    }
  }, []);

  return React__default.createElement(
    "div",
    { ref: wrapperRef, className: "\"react-transform-component\" " + styles.container },
    React__default.createElement(
      "div",
      { ref: contentRef, className: "react-transform-element " + styles.content, style: style },
      children
    )
  );
}

TransformComponent.propTypes = { children: PropTypes.any };

exports.TransformWrapper = TransformWrapper;
exports.TransformComponent = TransformComponent;
//# sourceMappingURL=index.js.map
